% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( Matrix )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [pyruvate_Matrix , COAS_Matrix , NAD_Matrix , CO2tot_Matrix , acetylcoA_Matrix , NADH_Matrix ]
%
%
% Free Parameters:
% [Vmf_PDH ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
% minimal parameter value
    MinCon = 0;


%% LIST OF STATE VARIABLES
% 1 pyruvate_Matrix
% 2 COAS_Matrix
% 3 NAD_Matrix
% 4 CO2tot_Matrix
% 5 acetylcoA_Matrix
% 6 NADH_Matrix
% 7 h_Matrix
% 8 m_Matrix
% 9 k_Matrix

% PARTIAL VOLUME FRACTIONS
VWater_Matrix = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_Matrix = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
pyruvate_Matrix = x(1);
COAS_Matrix = x(2);
NAD_Matrix = x(3);
CO2tot_Matrix = x(4);
acetylcoA_Matrix = x(5);
NADH_Matrix = x(6);
% Concentrations of H, Mg, and K
h_Matrix = x(7);
m_Matrix = x(8);
k_Matrix = x(9);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% pyruvate_Matrix
Kh(1) = 0.0035420488019067010963814023938311947858892381191254;
Km(1) = 0.095171974837220785214952911701402626931667327880859;
Kk(1) = 0.11946881075334832289502884350440581329166889190674;
% COAS_Matrix
Kh(2) = 7.5727660576433292701044864528477085219293485351955e-09;
Km(2) = Inf;
Kk(2) = Inf;
% NAD_Matrix
Kh(3) = Inf;
Km(3) = Inf;
Kk(3) = Inf;
% CO2tot_Matrix
Kh(4) = 1.5083741503259146546832528365555438354395079159076e-10;
Km(4) = Inf;
Kk(4) = Inf;
% acetylcoA_Matrix
Kh(5) = Inf;
Km(5) = Inf;
Kk(5) = Inf;
% NADH_Matrix
Kh(6) = Inf;
Km(6) = Inf;
Kk(6) = Inf;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_Matrix/Kh(1) + m_Matrix/Km(1) + k_Matrix/Kk(1);
P( 2 ) = 1  + h_Matrix/Kh(2) + m_Matrix/Km(2) + k_Matrix/Kk(2);
P( 3 ) = 1  + h_Matrix/Kh(3) + m_Matrix/Km(3) + k_Matrix/Kk(3);
P( 4 ) = 1  + h_Matrix/Kh(4) + m_Matrix/Km(4) + k_Matrix/Kk(4);
P( 5 ) = 1  + h_Matrix/Kh(5) + m_Matrix/Km(5) + k_Matrix/Kk(5);
P( 6 ) = 1  + h_Matrix/Kh(6) + m_Matrix/Km(6) + k_Matrix/Kk(6);

%% THERMODYNAMIC EQUATIONS
DGro_PDH =19.5952;

Keq_PDH_Matrix = exp(-DGro_PDH/RT)/P(1)/P(2)/P(3)*P(4)*P(5)*P(6)/h_Matrix;

%% FLUX EQUATIONS
%PDH_Matrix
a=pyruvate_Matrix;
b=COAS_Matrix;
c=NAD_Matrix;
p=CO2tot_Matrix;
q=acetylcoA_Matrix;
r=NADH_Matrix;
%KmandKivalues[M]
KmA=38.3e-6;
KmB=9.9e-6;
KmC=60.7e-6;
KiACCOA=40.2e-6;
KiNADH=40.0e-6;
Vmf_PDH=par(1);
%Inhibitionconstants
ai1=1+acetylcoA_Matrix/KiACCOA;
ai2=1+NADH_Matrix/KiNADH;
if(a>MinCon)&&(b>MinCon)&&(c>MinCon)
J_PDH_Matrix=Vmf_PDH*(a*b*c-p*q*r/Keq_PDH_Matrix)/(KmC*ai2*a*b+KmB*ai1*a*c+KmA*b*c+a*b*c);
else
J_PDH_Matrix=0;
end

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_PDH_Matrix ) / VWater_Matrix; % pyruvate_Matrix
f(2,:) = ( 0  - 1*J_PDH_Matrix ) / VWater_Matrix; % COAS_Matrix
f(3,:) = ( 0  - 1*J_PDH_Matrix ) / VWater_Matrix; % NAD_Matrix
f(4,:) = ( 0  + 1*J_PDH_Matrix ) / VWater_Matrix; % CO2tot_Matrix
f(5,:) = ( 0  + 1*J_PDH_Matrix ) / VWater_Matrix; % acetylcoA_Matrix
f(6,:) = ( 0  + 1*J_PDH_Matrix ) / VWater_Matrix; % NADH_Matrix

%% ION EQUATIONS
% COMPARTMENT Matrix:
ii = [1  2  3  4  5  6]; % Indices of SVs in compartment Matrix
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_Matrix./Km(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_Matrix./Kh(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_Matrix./Kh(ii)+m_Matrix./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 - 1*J_PDH_Matrix) / VWater_Matrix;
J_M = (0) / VWater_Matrix;
J_K = (0) / VWater_Matrix;
Phi_H = J_H - sum( h_Matrix*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_Matrix*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_Matrix*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_Matrix/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(7,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(8,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(9,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_PDH_Matrix];
